@@ -55,6 +55,7 @@ mpsigamma <- function(x,deriv=0,dim=1)
   return(PSI)
 }
 
+### S4 ###
 
 # forwarding function for list of a particular datatype
 zoom.list <- function(x,...)
@@ -66,6 +67,17 @@ zoom.list <- function(x,...)
 methods::setMethod("zoom",signature(x="list"), function(x,...) zoom.list(x,...))
 
 
+### S3 ###
+
+# forwarding function for list of a particular datatype
+log.list <- function(x,...)
+{
+  CLASS <- class(x[[1]])[1]
+  utils::getS3method("log",CLASS)(x,...)
+}
+# this doesn't work outside of ctmm
+
+
 # forwarding function for list of a particular datatype
 mean.list <- function(x,...)
 {
@@ -74,7 +86,6 @@ mean.list <- function(x,...)
 }
 #methods::setMethod("mean",signature(x="list"), function(x,...) mean.list(x,...))
 
-
 # forwarding function for list of a particular datatype
 median.list <- function(x,na.rm=FALSE,...)
 {
@@ -82,7 +93,6 @@ median.list <- function(x,na.rm=FALSE,...)
   utils::getS3method("median",CLASS)(x,na.rm=na.rm,...)
 }
 
-
 # forwarding function for list of a particular datatype
 plot.list <- function(x,...)
 {
@@ -260,3 +270,9 @@ copy <- function(from,to)
   for(n in NAMES){ to[[n]] <- from[[n]] }
   return(to)
 }
+
+mid <- function(x)
+{
+  n <- length(x)
+  (x[-1]+x[-n])/2
+}
