@@ -518,8 +518,9 @@ mc.optim <- function(par,fn,...,lower=-Inf,upper=Inf,period=FALSE,reset=identity
 
   is.toosmallf <- function(fns,fn0)
   {
-    if(ZERO) { abs(diff(fns))<=TOL.STAGE+TOL.ZERO }
-    else { abs(diff(fns))/fn0<=TOL.STAGE }
+    if(ZERO) { R <- abs(diff(fns))<=TOL.STAGE+TOL.ZERO }
+    else { R <- abs(diff(fns))/fn0<=TOL.STAGE }
+    return(!length(R) || is.na(R) || R) # avoid 0/0 error
   }
 
   is.toosmallp <- function(ps,p0,tol=STEP[STAGE])
@@ -527,7 +528,8 @@ mc.optim <- function(par,fn,...,lower=-Inf,upper=Inf,period=FALSE,reset=identity
     SCL <- pmax(abs(p0),1)
     # relative step sizes
     dp <- sqrt(abs(c((ps[,2]-ps[,1])^2 %*% (1/SCL^2)))) # formula explained in numderiv.diff()
-    return(dp <= tol)
+    R <- dp <= tol
+    return(!length(R) || is.na(R) || R) # avoid 0/0 error
   }
 
   numderiv.diff <- function(p0,DIR)
@@ -590,6 +592,7 @@ mc.optim <- function(par,fn,...,lower=-Inf,upper=Inf,period=FALSE,reset=identity
   ######################
   # MAIN LOOP
   ######################
+  if(trace==1) { pb <- utils::txtProgressBar(style=3) }
   while(STAGE<=last(stages) && counts<=maxit)
   {
     # udpate zero shift
@@ -944,6 +947,7 @@ mc.optim <- function(par,fn,...,lower=-Inf,upper=Inf,period=FALSE,reset=identity
       # evaluate objective function at new P and store to fn.queue
       fn.queue <- unlist(plapply(split(P,col(P)),func,cores=cores))
       PROGRESS <- any(fn.queue<fn.par) # did this iteration improve things?
+      PROGRESS <- is.good(PROGRESS) # possible 0/0 error somewhere
 
       # combine with older results
       par.all <- cbind(par.all,P)
@@ -962,8 +966,8 @@ mc.optim <- function(par,fn,...,lower=-Inf,upper=Inf,period=FALSE,reset=identity
 
       END <- (MIN==1 || MIN==length(fn.all))
 
-      if(trace) { message(sprintf("%s %s search",format(zero+fn.par,digits=16),LINE.TYPE)) }
-      if(trace>1) { message("\tc(",paste0(NAMES,"=",par,collapse=', '),")") }
+      if(trace==2) { message(sprintf("%s %s search",format(zero+fn.par,digits=16),LINE.TYPE)) }
+      if(trace==3) { message("\tc(",paste0(NAMES,"=",par,collapse=', '),")") }
 
       # if(DEBUG>1)
       # {
@@ -1218,11 +1222,13 @@ mc.optim <- function(par,fn,...,lower=-Inf,upper=Inf,period=FALSE,reset=identity
       }
     }
 
+    if(trace==1) { utils::setTxtProgressBar(pb,clamp(nant(log(ERROR)/log(TOL.GOAL),0))) }
   } # end main loop
+  if(trace==1) { close(pb) }
 
   if(counts<maxit) { convergence <- 0} else { convergence <- 1 }
   if(trace) { message(sprintf("%s in %d parallel function evaluations.",ifelse(convergence,"No convergence","Convergence"),counts)) }
-  if(trace==2) { message("\tc(",paste0(NAMES,"=",par,collapse=', '),")") }
+  if(trace==3) { message("\tc(",paste0(NAMES,"=",par,collapse=', '),")") }
 
   if(PMAP) { par <- pmap(par,theta,inverse=TRUE) }
 
