@@ -13,11 +13,6 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
 
   CALC <- integrated || length(R) # anything to calculate?
 
-  # smooth the data, but don't drop
-  if(smooth && any(CTMM$error>0))
-  { data[,c(axes,GEO)] <- predict(data,CTMM=CTMM,t=data$t,complete=TRUE)[,c(axes,GEO)] }
-  n <- nrow(data)
-
   if(!integrated) # prepare available region polygon
   {
     if(class(level.UD)[1]=="numeric")
@@ -30,13 +25,18 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
     { level.UD <- sp::spTransform(level.UD,sp::CRS(projection(data))) }
     AREA <- level.UD@area
   }
-  else if(isotropic && !CTMM$isotropic) # keep things simple for now
+  else if(isotropic && !CTMM$isotropic) # until rsf.select() is coded
   {
     message('RSF code is isotropic for the moment.')
     CTMM <- simplify.ctmm(CTMM,'minor')
     CTMM <- ctmm.fit(data,CTMM,trace=trace)
   }
 
+  # smooth the data, but don't drop
+  if(smooth && any(CTMM$error>0))
+  { data[,c(axes,GEO)] <- predict(data,CTMM=CTMM,t=data$t,complete=TRUE)[,c(axes,GEO)] }
+  n <- nrow(data)
+
   # for simulation - uncorrelated, error-less
   IID <- CTMM
   IID$tau <- NULL
@@ -58,14 +58,27 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
     names(R) <- paste("R",1:length(R))
   }
 
-  # how to sample rasters
-  interpolate <- rep(interpolate,length(R))
-  interpolate <- ifelse(interpolate,"bilinear","simple")
-  names(interpolate) <- names(R)
+  # expand any categorical variables into indicator variables
+  if(length(R))
+  {
+    if(!is.null(formula))
+    {
+      RVARS <- names(R) # this will be updated
+      VARS <- all.vars(formula) # this will be updated
+      DVARS <- VARS[ VARS %nin% RVARS ]
+    }
+    else
+    { DVARS <- NULL }
+
+    STUFF <- expand.factors(R=R,formula=formula,reference=reference,data=data,DVARS)
+    data <- STUFF$data
+    R <- STUFF$R
+    formula <- STUFF$formula
+  }
 
   if(!is.null(formula) && standardize)
   {
-    message("Users are responsible for standardizing ",names(R)," when formula argument is supplied.")
+    message("Users are responsible for standardizing rasters when formula argument is supplied.")
     standardize <- FALSE
   }
   else
@@ -74,13 +87,10 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
     names(RSCALE) <- names(R)
   }
 
-  # expand any categorical variables into indicator variables
-  if(length(R))
-  {
-    STUFF <- expand.factors(R=R,formula=formula,reference=reference,data=data)
-    R <- STUFF$R
-    formula <- STUFF$formula
-  }
+  # how to sample rasters
+  interpolate <- rep(interpolate,length(R))
+  interpolate <- ifelse(interpolate,"bilinear","simple")
+  names(interpolate) <- names(R)
 
   ## The basic setup is:
   # RVARS - raster variables
@@ -112,9 +122,16 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
     VARS <- all.vars(formula)
     DVARS <- VARS[ VARS %nin% RVARS ]
 
-    TERMS <- attr(stats::terms(formula),"term.labels")
+    # this doesn't work with poly(), etc.
+    # TERMS <- attr(stats::terms(formula),"term.labels")
+    # dummy data for model parameter names
+    DATA <- data.frame(data)
+    DATA[RVARS] <- as.list(rep(0,length(RVARS)))
+    DATA[['rr']] <- 0
+    TERMS <- colnames(stats::model.matrix(formula,data=DATA))
+    TERMS <- TERMS[TERMS!="(Intercept)"]
 
-    CVARS <- TERMS[ TERMS %nin% VARS ] # remaining terms that are not simple variables
+    CVARS <- TERMS[ TERMS %nin% VARS ] # terms that are not simple variables
 
     OFFSET <- stats::terms(formula)
     OFFSET <- attr(OFFSET,"variables")[ attr(OFFSET,"offset") ]
@@ -134,17 +151,19 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
   }
 
   # evaluate term on data.frame
-  evaluate <- function(term,envir)
+  evaluate <- function(term,envir,offset=FALSE)
   {
-    if(length(dim(envir))==2)
+    if(length(dim(envir))==2) # (term,DATA[time,var])
     {
-      term <- gsub(":","*",term) # multiplication
+      # term <- gsub(":","*",term) # multiplication
       # term <- gsub("I(","(",term) # function evaluations # don't think this is necessary
       envir <- data.frame(envir) # matrices can't be environments, but data.frames can't matrix multiply...
       if(!STATIONARY) { envir <- cbind(envir,data) }
-      RET <- eval(parse(text=term),envir=envir)
+      # RET <- eval(parse(text=term),envir=envir)
+      RET <- stats::model.matrix(formula,envir)[,term]
+      if(offset) { RET <- apply(RET,1,prod) }
     }
-    else # loop over runs
+    else # [space,time,var] loop over runs
     {
       if(STATIONARY) # time doesn't matter
       {
@@ -152,20 +171,20 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
         dim(envir) <- c(DIM[1]*DIM[2],DIM[3])
         colnames(envir) <- VARS
         RET <- evaluate(term,envir)
-        dim(RET) <- DIM[1:2]
+        dim(RET) <- c(DIM[1:2],length(term))
       }
       else # !STATIONARY
       {
         RET <- sapply(1:dim(envir)[1],function(i)
                 {
-                  ENVIR <- envir[i,,];
+                  ENVIR <- envir[i,,]; # [time,var]
                   dim(ENVIR) <- dim(envir)[-1];
                   colnames(ENVIR) <- VARS
-                  ENVIR <- cbind(ENVIR,data);
+                  ENVIR <- cbind(ENVIR,data); # [time,vars]
                   evaluate(term,ENVIR)
-                }) # [time,track]
-        dim(RET) <- dim(envir)[2:1]
-        RET <- t(RET) # [track,time]
+                }) # [time,space,terms]
+        dim(RET) <- c(nrow(data),dim(envir)[1],length(term))
+        RET <- aperm(RET,c(2,1,3)) # [space,time,terms]
       }
     }
     return(RET)
@@ -235,14 +254,12 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
 
     if(length(R)>1)
     {
-      if(any(diff(dX)!=0) || any(diff(dY)!=0)) # check for consistent resolution
-      { CONSISTENT <- FALSE }
-      else if(any(diff(Xo)!=0) || any(diff(Yo)!=0)) # check for consistent origin
+      if(max(abs(c(diff(dX)/mid(dX),diff(dY)/mid(dY),diff(Xo)/mid(Xo),diff(Yo)/mid(Yo))))>.Machine$double.eps) # check for consistent resolution, origin
       { CONSISTENT <- FALSE }
     }
 
     if(CONSISTENT) # extract pixel areas for integration
-    { dA <- raster::area(R[[1]]) }
+    { dA <- raster::area(R[[1]]) } # TODO generalize this for projected covariates
     else # choose minimum resolution for integration grid
     {
       dx <- min(UD$dr['x'],dX)
@@ -289,12 +306,21 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
   DATA <- matrix(0,nrow(data),length(VARS))
   colnames(DATA) <- VARS
 
-  # # generic SCALE code for arbitrary formula terms not possible
-  # SCALES <- c(RVARS,DVARS)
-  # SCALES <- t(SCALES)
-  # SCALES <- as.data.frame(SCALES)
-  # SCALE <- rep(0,length(TERMS))
-  # for(i in 1:length(TERMS)) { SCALE[i] <- evaluate(TERMS[i],SCALES) }
+  # natural scales for differentiation/optimization
+  parscale <- rep(1,length(TERMS))
+  lower <- rep(-Inf,length(TERMS))
+  upper <- rep(Inf,length(TERMS))
+  names(parscale) <- names(lower) <- names(upper) <- TERMS
+
+  if(integrated)
+  {
+    LOV <- ifelse(integrator=="MonteCarlo",-1,0)
+
+    if(isotropic)
+    { lower['rr'] <- LOV }
+    else
+    { lower[c('xx','yy')] <- LOV }
+  }
 
   # initial estimates
   BETA <- rep(0,length(TERMS))
@@ -345,7 +371,7 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
   }
 
   # calculate and store composite covariates
-  for(v in CVARS) { DATA[,v] <- evaluate(v,DATA) }
+  DATA[,CVARS] <- evaluate(CVARS,DATA)
 
   ### store sampled integrated terms
   if(integrated)
@@ -386,20 +412,38 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
   } # end if(integrated)
 
   DAVE <- c(w %*% DATA)
-  if(any(is.na(DAVE))) { stop("NA values in sampled rasters.") }
   names(DAVE) <- VARS
+  NAS <- is.na(DAVE)
+  if(any(NAS))
+  {
+    VARS <- paste0(VARS[NAS],collapse=",")
+
+    IND <- rowSums(is.na(DATA))
+    HEAD <- head(IND)
+    if(length(HEAD)<length(IND))
+    { HEAD <- paste0(HEAD,",...",collapse=",") }
+    else
+    { HEAD <- paste0(HEAD,collapse=",") }
+
+    STOP <- paste0("NA values in sampled variables ",VARS," at points ",HEAD)
+    stop(STOP)
+  }
+
   SATA <- array(0,c(0,dim(DATA))) # simulated data [track,time,vars]
   dimnames(SATA)[[3]] <- VARS
 
   nloglike <- function(beta,zero=0,verbose=FALSE)
   {
-    SAMP <- exp(SATA[,,TERMS,drop=FALSE] %.% beta) # [track,time]
+    SAMP <- SATA[,,TERMS,drop=FALSE] %.% beta # [track,time]
+    SHIFT <- mean(SAMP,na.rm=TRUE)
+    SAMP <- SAMP - SHIFT
+    SAMP <- exp(SAMP) # + exp(SHIFT)
     SAMP[] <- nant(SAMP,0) # treat NA as inaccessible region
 
     if(length(OFFSET))
     {
       ONE <- rep(TRUE,nrow(SATA))
-      for(o in OFFSET) { ONE <- ONE & evaluate(o,SATA) }
+      ONE <- evaluate(OFFSET,SATA,offset=TRUE)
       SAMP <- ONE * SAMP
     }
 
@@ -414,7 +458,7 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
       else # Newton integration
       { MEAN <- sum(SAMP*dA) }
 
-      log.MEAN <- log(MEAN)
+      log.MEAN <- log(MEAN) + SHIFT
 
       if(integrator=="MonteCarlo")
       {
@@ -436,7 +480,7 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
     else # !STATIONARY
     {
       MEAN <- apply(SAMP,2,mean) # [time]
-      log.MEAN <- log(MEAN)
+      log.MEAN <- log(MEAN) + SHIFT
 
       if(debias || verbose) # numerical error variance (per w^2)
       { VAR.log <- apply(SAMP,2,stats::var)/dim(SAMP)[1] /MEAN^2 }
@@ -558,8 +602,10 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
       else # elliptical grid
       { stop('Inconsistent grids not yet supported when method!="MonteCarlo".') }
 
-      # This is currently coded for stationary only
-      SATA <- array(0,c(N,1,length(VARS)))
+      if(STATIONARY)
+      { SATA <- array(0,c(N,1,length(VARS))) }
+      else
+      { SATA <- array(0,c(N,nrow(data),length(VARS))) }
     } # end quadrature points
 
     dimnames(SATA)[[3]] <- VARS
@@ -604,7 +650,7 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
       }
     }
 
-    for(v in CVARS) { SATA[SUB,,v] <- evaluate(v,SATA[SUB,,,drop=FALSE]) }
+    SATA[SUB,,CVARS] <- evaluate(CVARS,SATA[SUB,,,drop=FALSE])
 
     if(integrated)
     {
@@ -658,7 +704,7 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
       { beta <- beta.init }
     }
 
-    RESULT <- optimizer(beta,nloglike,control=control)
+    RESULT <- optimizer(beta,nloglike,parscale=parscale,lower=lower,upper=upper,control=control)
     beta.OLD <- beta
     beta <- RESULT$par
     loglike.OLD <- loglike
@@ -702,7 +748,7 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
   if(CALC)
   {
     if(trace) { message("Calculating Hessian") }
-    DIFF <- genD(par=beta,fn=nloglike,zero=-loglike,Richardson=2,mc.cores=1)
+    DIFF <- genD(par=beta,fn=nloglike,zero=-loglike,parscale=parscale,lower=lower,upper=upper,Richardson=2,mc.cores=1)
     hess <- DIFF$hessian
     grad <- DIFF$gradient
     # more robust covariance calculation than straight inverse
@@ -766,7 +812,13 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
       return(beta)
     }
 
-    J <- genD(par=beta,fn=fn,order=1,drop=FALSE)$gradient
+    # fix parscale for Jacobian
+    parscale[names(SCALE)] <- 1/SCALE
+    EX <- names(parscale)
+    EX <- EX[EX %nin% names(SCALE)]
+    if(length(EX)) { parscale[EX] <- sqrt(diag(COV)[EX]) }
+
+    J <- genD(par=beta,fn=fn,order=1,drop=FALSE,parscale=parscale,lower=lower,upper=upper)$gradient
     beta <- fn(beta)
     COV <- J %*% COV %*% t(J)
 
@@ -795,7 +847,7 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
     if(!is.null(CTMM$MLE))
     { DEBIAS <- sqrt(det.covm(CTMM$sigma)/det.covm(CTMM$MLE$sigma)) }
     else
-    { DEBIAS <- W/(W-1) }
+    { DEBIAS <- max(W,2)/max(W-1,1) }
 
     DSCALE <- rep(1,length(TERMS))
     names(DSCALE) <- TERMS
@@ -913,10 +965,10 @@ rsf.fit <- function(data,UD,beta=NULL,R=list(),formula=NULL,integrated=TRUE,refe
 
 
 # expand raster factors into
-expand.factors <- function(R,formula,reference="auto",data=NULL,fixed=FALSE)
+expand.factors <- function(R,formula,reference="auto",data=NULL,DVARS=NULL,fixed=FALSE)
 {
   NAMES <- names(R)
-  for(i in 1:length(R))
+  for(i in 1%:%length(R))
   {
     if(raster::is.factor(R[[i]]))
     {
@@ -943,21 +995,22 @@ expand.factors <- function(R,formula,reference="auto",data=NULL,fixed=FALSE)
       else if(reference=="auto") # fix base layer
       {
         PROJ <- raster::projection(FACT)
-        data <- get.telemetry(data,c("longitude","latitude"))
-        colnames(data) <- c('x','y')
-        data <- project(data,to=PROJ)
-        data <- raster::extract(FACT,data) # don't interpolate
-        UNIQUE <- unique(data) # may be missing some levels
-        data <- tabulate(match(data,UNIQUE)) # tallies
-        reference <- UNIQUE[which.max(data)]
-        message("Reference category set to ",reference,".")
+        XY <- get.telemetry(data,c("longitude","latitude"))
+        colnames(XY) <- c('x','y')
+        XY <- project(XY,to=PROJ)
+        XY <- raster::extract(FACT,XY) # don't interpolate
+        UNIQUE <- unique(XY) # may be missing some levels
+        XY <- tabulate(match(XY,UNIQUE)) # tallies
+        reference <- UNIQUE[which.max(XY)]
+        message(NAME," reference category set to ",reference,".")
       }
       # else the reference category is specified by `reference`
 
+      REF <- reference[reference %in% LEVELS]
       DIFF <- LEVELS %nin% reference
       FACT <- lapply(LEVELS[DIFF],function(l){FACT==l})
       # LEVELS <- paste0(NAME,"[",LEVELS,"/",LEVELS[reference],"]")[DIFF]
-      LEVELS <- paste0(NAME,".",LEVELS,"_",LEVELS[reference])[DIFF]
+      LEVELS <- paste0(NAME,".",LEVELS,"_",REF)[DIFF]
       names(FACT) <- LEVELS
 
       R <- c(R,FACT)
@@ -973,10 +1026,67 @@ expand.factors <- function(R,formula,reference="auto",data=NULL,fixed=FALSE)
         formula <- simplify.formula(formula)
       }
     }
-  }
-  if(fixed) { return(R) }
+  } # end raster expansion
+
+  NAMES <- DVARS
+  for(i in 1%:%length(DVARS))
+  {
+    NAME <- NAMES[i]
+    if(is.factor(data[[NAME]]))
+    {
+      FACT <- data[[NAME]]
+      KEEP <-
+      data <- data[names(data)!=NAME]
+      NAMES <- NAMES[NAMES!=NAME]
+
+      LEVELS <- levels(FACT) # assuming one layer
+
+      if(fixed)
+      {
+        TERMS <- attr(stats::terms(formula),"term.labels")
+        # I am not good with regexp
+        # HEAD <- paste0(NAME,"[")
+        HEAD <- paste0(NAME,".")
+        reference <- TERMS[grepl(HEAD,TERMS,fixed=TRUE)][1] # *NAME[#/ref]*
+        reference <- strsplit(reference,HEAD,fixed=TRUE)[[1]][2] # #/ref]*
+        # reference <- strsplit(reference,"/",fixed=TRUE)[[1]][2] # ref]*
+        reference <- strsplit(reference,"_",fixed=TRUE)[[1]][2] # ref]*
+        # reference <- strsplit(reference,"]",fixed=TRUE)[[1]][1] # ref
+        reference <- which(LEVELS==reference)
+      }
+      else if(reference=="auto") # fix base layer
+      {
+        UNIQUE <- unique(FACT) # may be missing some levels
+        XY <- tabulate(match(FACT,UNIQUE)) # tallies
+        reference <- UNIQUE[which.max(XY)]
+        message(NAME," reference category set to ",reference,".")
+      }
+      # else the reference category is specified by `reference`
+
+      REF <- reference[reference %in% LEVELS]
+      DIFF <- LEVELS %nin% reference
+      FACT <- lapply(LEVELS[DIFF],function(l){FACT==l})
+      # LEVELS <- paste0(NAME,"[",LEVELS,"/",LEVELS[reference],"]")[DIFF]
+      LEVELS <- paste0(NAME,".",LEVELS,"_",REF)[DIFF]
+      names(FACT) <- LEVELS
+
+      data[LEVELS] <- FACT
+
+      # expand terms
+      if(!fixed && !is.null(formula))
+      {
+        formula <- as.character(formula)[2]
+        formula <- sapply(LEVELS,function(l){gsub(NAME,l,formula,fixed=TRUE)})
+        formula <- paste(formula,collapse="+")
+        formula <- paste("~",formula)
+        formula <- eval(parse(text=formula))
+        formula <- simplify.formula(formula)
+      }
+    }
+  } # end telemetry expansion
+
   environment(formula) <- NULL
-  RETURN <- list(R=R,formula=formula)
+  RETURN <- list(data=data,R=R,formula=formula)
 }
 
 
